@@ -1,3 +1,7 @@
+# coding: utf-8
+
+from __future__ import unicode_literals
+
 """
 This module contains the base class for implementing Duplicate Finders
 """
@@ -43,4 +47,4 @@ class DupeFinderBase(FWSerializable):
 
     @classmethod
     def from_dict(cls, m_dict):
-        return cls()
\ No newline at end of file
+        return cls()
