@@ -95,6 +95,9 @@ def _recursive_load(obj):
     if obj is None:
         return None
 
+    if hasattr(obj, '_fw_name'):
+        return obj
+
     if isinstance(obj, dict):
         if '_fw_name' in obj:
             return load_object(obj)
@@ -158,7 +161,7 @@ def serialize_fw(func):
     return _decorator
 
 
-six.add_metaclass(abc.ABCMeta)
+@six.add_metaclass(abc.ABCMeta)
 class FWSerializable(object):
     """
     To create a serializable object within FireWorks, you should subclass this
@@ -309,7 +312,7 @@ def load_object(obj_dict):
         for loader, mod_name, is_pkg in pkgutil.walk_packages(
                 root_module.__path__, package + '.'):
             try:
-                m_module = loader.find_module(mod_name).load_module(mod_name)
+                m_module = importlib.import_module(mod_name)
                 m_object = _search_module_for_obj(m_module, obj_dict)
                 if m_object is not None:
                     found_objects.append((m_object, mod_name))
